@@ -6,9 +6,11 @@ use std::sync::atomic::AtomicU64;
 use std::sync::atomic::Ordering;
 use std::time::Instant;
 
+use syndicate::language;
 use syndicate::actor::*;
 use syndicate::during::entity;
 use syndicate::dataspace::Dataspace;
+use syndicate::enclose;
 use syndicate::schemas::dataspace::Observe;
 use syndicate::schemas::dataspace_patterns as p;
 use syndicate::value::Map;
@@ -59,17 +61,16 @@ pub fn bench_pub(c: &mut Criterion) {
                     let account = Account::new(syndicate::name!("sender-account"));
                     t.linked_task(syndicate::name!("sender"), async move {
                         for _ in 0..iters {
-                            let ds = Arc::clone(&ds);
-                            external_event(&Arc::clone(&ds.mailbox), &account, Box::new(
-                                move |t| t.with_entity(
+                            external_event(&ds.mailbox, &account, Box::new(
+                                enclose!((ds) move |t| t.with_entity(
                                     &ds,
                                     |t, e| e.message(t, says(AnyValue::new("bench_pub"),
-                                                             Value::ByteString(vec![]).wrap())))))?
+                                                             Value::ByteString(vec![]).wrap()))))))?
                         }
-                        external_event(&Arc::clone(&shutdown.mailbox), &account, Box::new(
-                            move |t| t.with_entity(
+                        external_event(&shutdown.mailbox, &account, Box::new(
+                            enclose!((shutdown) move |t| t.with_entity(
                                 &shutdown,
-                                |t, e| e.message(t, AnyValue::new(true)))))?;
+                                |t, e| e.message(t, AnyValue::new(true))))))?;
                         Ok(())
                     });
                     Ok(())
@@ -101,7 +102,7 @@ pub fn bench_pub(c: &mut Criterion) {
                             })
                             .create_cap(t);
 
-                        ds.assert(t, &Observe {
+                        ds.assert(t, language(), &Observe {
                             pattern: p::Pattern::DBind(Box::new(p::DBind {
                                 pattern: p::Pattern::DLit(Box::new(p::DLit {
                                     value: AnyValue::symbol("consumer"),
@@ -122,8 +123,8 @@ pub fn bench_pub(c: &mut Criterion) {
                             let shutdown = Cap::new(&t.create(ShutdownEntity));
                             let receiver = Cap::new(&t.create(Receiver(Arc::clone(&turn_count))));
 
-                            ds.assert(t, Value::<AnyValue, _>::symbol("consumer").wrap());
-                            ds.assert(t, &Observe {
+                            ds.assert(t, &(), &AnyValue::symbol("consumer"));
+                            ds.assert(t, language(), &Observe {
                                 pattern: p::Pattern::DCompound(Box::new(p::DCompound::Rec {
                                     ctor: Box::new(p::CRec {
                                         label: AnyValue::symbol("Says"),
@@ -140,7 +141,7 @@ pub fn bench_pub(c: &mut Criterion) {
                                 })),
                                 observer: receiver,
                             });
-                            ds.assert(t, &Observe {
+                            ds.assert(t, language(), &Observe {
                                 pattern: p::Pattern::DBind(Box::new(p::DBind {
                                     pattern: p::Pattern::DLit(Box::new(p::DLit {
                                         value: AnyValue::new(true),
