@@ -5,24 +5,30 @@
     include_str!("../doc/linked-tasks.md"),
 )]
 
-use super::schemas::sturdy;
+use super::dataflow::Graph;
 use super::error::Error;
 use super::error::encode_error;
 use super::error::error;
 use super::rewrite::CaveatError;
 use super::rewrite::CheckedCaveat;
+use super::schemas::sturdy;
 
+use preserves::value::ArcValue;
 use preserves::value::Domain;
 use preserves::value::IOValue;
 use preserves::value::Map;
 use preserves::value::NestedValue;
 use preserves::value::Set;
-use preserves_schema::support::ParseError;
+use preserves_schema::ParseError;
+use preserves_schema::support::Parse;
+use preserves_schema::support::Unparse;
 
+use std::any::Any;
 use std::boxed::Box;
 use std::collections::hash_map::HashMap;
 use std::convert::TryFrom;
 use std::convert::TryInto;
+use std::marker::PhantomData;
 use std::num::NonZeroU64;
 use std::sync::Arc;
 use std::sync::Mutex;
@@ -48,7 +54,7 @@ use tracing::Instrument;
 /// `M`-values can be exchanged among objects, for distributed or
 /// polyglot systems a *lingua franca* has to be chosen. `AnyValue` is
 /// that language.
-pub type AnyValue = super::schemas::internal_protocol::_Any;
+pub type AnyValue = ArcValue<Arc<Cap>>;
 
 /// The type of process-unique actor IDs.
 pub type ActorId = u64;
@@ -63,6 +69,12 @@ pub type FacetId = NonZeroU64;
 /// local state with some incoming assertion in an entity.
 pub type Handle = u64;
 
+/// The type of process-unique field instance IDs.
+pub type FieldId = NonZeroU64;
+
+/// The type of process-unique field observer block IDs.
+pub type BlockId = NonZeroU64;
+
 /// Responses to events must have type `ActorResult`.
 pub type ActorResult = Result<(), Error>;
 
@@ -173,7 +185,7 @@ pub trait Entity<M>: Send {
     /// Optional callback for running actions when the entity's owning [Facet] terminates
     /// cleanly. Will not be called in case of abnormal shutdown (crash) of an actor.
     ///
-    /// Programs register an entity's stop hook with [Activation::on_stop].
+    /// Programs register an entity's stop hook with [Activation::on_stop_notify].
     ///
     /// The default implementation does nothing.
     fn stop(&mut self, turn: &mut Activation) -> ActorResult {
@@ -206,6 +218,7 @@ enum CleanupAction {
 type CleanupActions = Map<Handle, CleanupAction>;
 
 type Action = Box<dyn Send + FnOnce(&mut Activation) -> ActorResult>;
+type Block = Box<dyn Send + FnMut(&mut Activation) -> ActorResult>;
 
 #[doc(hidden)]
 pub type PendingEventQueue = Vec<Action>;
@@ -224,13 +237,14 @@ pub type PendingEventQueue = Vec<Action>;
 /// values contained in an `Activation` are also sometimes useful.
 ///
 /// This is what other implementations call a "Turn", renamed here to
-/// avoid conflicts with [`crate::schemas::internal_protocol::Turn`].
+/// avoid conflicts with [`crate::schemas::protocol::Turn`].
 pub struct Activation<'activation> {
     /// A reference to the currently active [`Facet`] and the implementation-side state of its
     /// [`Actor`].
     pub facet: FacetRef,
     /// A reference to the current state of the active [`Actor`].
     pub state: &'activation mut RunningActor,
+    active_block: Option<BlockId>,
     pending: EventBuffer,
 }
 
@@ -269,6 +283,7 @@ pub struct LoanedItem<T> {
 
 enum SystemMessage {
     Release,
+    ReleaseField(FieldId),
     Turn(LoanedItem<PendingEventQueue>),
     Crash(Error),
 }
@@ -323,6 +338,9 @@ pub struct RunningActor {
     pub actor_id: ActorId,
     tx: UnboundedSender<SystemMessage>,
     mailbox: Weak<Mailbox>,
+    dataflow: Graph<FieldId, BlockId>,
+    fields: HashMap<FieldId, Box<dyn Any + Send>>,
+    blocks: HashMap<BlockId, (FacetId, Block)>,
     exit_hooks: Vec<Box<dyn Send + FnOnce(&mut Activation, &Arc<ActorResult>) -> ActorResult>>,
     cleanup_actions: CleanupActions,
     facet_nodes: Map<FacetId, Facet>,
@@ -330,6 +348,20 @@ pub struct RunningActor {
     root: FacetId,
 }
 
+/// Handle to a shared, mutable field (i.e. a *dataflow variable*) within a [`RunningActor`].
+///
+/// Use [`Activation::field`] to create fields, and use [`Activation::get`],
+/// [`::get_mut`][Activation::get_mut], and [`::set`][Activation::set] to read and write field
+/// values. Use [`Activation::dataflow`] to create a reactive block within a facet that will be
+/// (re-)executed whenever some dependent field changes value.
+///
+#[derive(Debug)]
+pub struct Field<T: Any + Send> {
+    pub field_id: FieldId,
+    tx: UnboundedSender<SystemMessage>,
+    phantom: PhantomData<T>,
+}
+
 /// State associated with each facet in an [`Actor`]'s facet tree.
 ///
 /// # Inert facets
@@ -401,18 +433,26 @@ pub struct Cap {
 /// The [`Entity`] implementation for `Guard` decodes `AnyValue`
 /// assertions/messages to type `M` before passing them on to the
 /// underlying entity.
-pub struct Guard<M>
+pub struct Guard<L, M>
 where
-    for<'a> &'a M: Into<AnyValue>,
-    for<'a> M: TryFrom<&'a AnyValue>,
+    M: for<'a> Unparse<&'a L, AnyValue>,
+    M: for<'a> Parse<&'a L, AnyValue>,
 {
-    underlying: Arc<Ref<M>>
+    underlying: Arc<Ref<M>>,
+    literals: Arc<L>,
 }
 
 /// Simple entity that stops its containing facet when any assertion it receives is
 /// subsequently retracted.
 pub struct StopOnRetract;
 
+/// Returned from the function given to [`FacetRef::activate_exit`] to indicate how the actor
+/// should proceed.
+pub enum RunDisposition {
+    Continue,
+    Terminate(ActorResult),
+}
+
 //---------------------------------------------------------------------------
 
 const BUMP_AMOUNT: u8 = 10;
@@ -440,6 +480,10 @@ static NEXT_ACCOUNT_ID: AtomicU64 = AtomicU64::new(4);
 
 static NEXT_TASK_ID: AtomicU64 = AtomicU64::new(5);
 
+static NEXT_FIELD_ID: AtomicU64 = AtomicU64::new(6);
+
+static NEXT_BLOCK_ID: AtomicU64 = AtomicU64::new(7);
+
 preserves_schema::support::lazy_static! {
     #[doc(hidden)]
     pub static ref SYNDICATE_CREDIT: i64 = {
@@ -472,6 +516,27 @@ impl From<&Synced> for AnyValue {
     }
 }
 
+impl<'a> Parse<&'a (), AnyValue> for Synced {
+    fn parse(_language: &'a (), value: &AnyValue) -> Result<Self, ParseError> {
+        Synced::try_from(value)
+    }
+}
+
+impl<'a> Unparse<&'a (), AnyValue> for Synced {
+    fn unparse(&self, _language: &'a ()) -> AnyValue {
+        self.into()
+    }
+}
+
+impl From<ActorResult> for RunDisposition {
+    fn from(v: ActorResult) -> Self {
+        match v {
+            Ok(()) => RunDisposition::Continue,
+            Err(e) => RunDisposition::Terminate(Err(e)),
+        }
+    }
+}
+
 impl FacetRef {
     /// Executes `f` in a new "[turn][Activation]" for `actor`. If `f` returns `Ok(())`,
     /// [commits the turn][Activation::deliver] and performs the buffered actions; otherwise,
@@ -485,13 +550,7 @@ impl FacetRef {
     ) -> ActorResult where
         F: FnOnce(&mut Activation) -> ActorResult,
     {
-        match self.activate_exit(account, |t| match f(t) {
-            Ok(()) => None,
-            Err(e) => Some(Err(e)),
-        }) {
-            None => Ok(()),
-            Some(e) => Err(error("Could not activate terminated actor", encode_error(e))),
-        }
+        self.activate_exit(account, |t| f(t).into())
     }
 
     /// Executes `f` in a new "[turn][Activation]" for `actor`. If `f` returns
@@ -504,20 +563,22 @@ impl FacetRef {
         &self,
         account: Arc<Account>,
         f: F,
-    ) -> Option<ActorResult> where
-        F: FnOnce(&mut Activation) -> Option<ActorResult>,
+    ) -> ActorResult where
+        F: FnOnce(&mut Activation) -> RunDisposition,
     {
         match self.actor.state.lock() {
             Err(_) => panicked_err(),
             Ok(mut g) => match &mut *g {
                 ActorState::Terminated { exit_status } =>
-                    Some((**exit_status).clone()),
+                    Err(error("Could not activate terminated actor",
+                              encode_error((**exit_status).clone()))),
                 ActorState::Running(state) => {
                     tracing::trace!(actor_id=?self.actor.actor_id, "activate");
                     let mut activation = Activation::make(self, account, state);
-                    let result = match f(&mut activation) {
-                        None => None,
-                        Some(exit_status) => {
+                    let f_result = f(&mut activation);
+                    let result = match activation.restore_invariants(f_result) {
+                        RunDisposition::Continue => Ok(()),
+                        RunDisposition::Terminate(exit_status) => {
                             if exit_status.is_err() {
                                 activation.clear();
                             }
@@ -542,7 +603,7 @@ impl FacetRef {
                             *g = ActorState::Terminated {
                                 exit_status: Arc::clone(&exit_status),
                             };
-                            Some((*exit_status).clone())
+                            (*exit_status).clone()
                         }
                     };
                     tracing::trace!(actor_id=?self.actor.actor_id, "deactivate");
@@ -562,6 +623,7 @@ impl<'activation> Activation<'activation> {
         Activation {
             facet: facet.clone(),
             state,
+            active_block: None,
             pending: EventBuffer::new(account),
         }
     }
@@ -684,6 +746,22 @@ impl<'activation> Activation<'activation> {
         }
     }
 
+    /// Core API: assert, retract, or replace an assertion.
+    pub fn update<M: 'static + Send + std::fmt::Debug>(
+        &mut self,
+        handle: &mut Option<Handle>,
+        r: &Arc<Ref<M>>,
+        a: Option<M>,
+    ) {
+        let saved = handle.take();
+        if let Some(a) = a {
+            *handle = Some(self.assert(r, a));
+        }
+        if let Some(h) = saved {
+            self.retract(h);
+        }
+    }
+
     /// Core API: send message `m` to recipient `r`.
     pub fn message<M: 'static + Send + std::fmt::Debug>(&mut self, r: &Arc<Ref<M>>, m: M) {
         tracing::trace!(?r, ?m, "message");
@@ -726,18 +804,27 @@ impl<'activation> Activation<'activation> {
     /// Registers the entity `r` in the list of stop actions for the active facet. If the facet
     /// terminates cleanly, `r`'s [`stop`][Entity::stop] will be called.
     ///
-    /// **Note.** If the actor crashes, the stop actions will *not* be called.
+    /// **Note.** If the actor crashes, stop actions will *not* be called.
     ///
     /// Use [`RunningActor::add_exit_hook`] to install a callback that will be called at the
     /// end of the lifetime of the *actor* rather than the facet. (Also, exit hooks are called
     /// no matter whether actor termination was normal or abnormal.)
-    pub fn on_stop<M: 'static + Send>(&mut self, r: &Arc<Ref<M>>) {
+    pub fn on_stop_notify<M: 'static + Send>(&mut self, r: &Arc<Ref<M>>) {
         if let Some(f) = self.active_facet() {
             let r = Arc::clone(r);
             f.stop_actions.push(Box::new(move |t| r.internal_with_entity(|e| e.stop(t))));
         }
     }
 
+    /// Registers `action` in the list of stop actions for the active facet. If the facet
+    /// terminates cleanly, `action` will be called. See also notes against
+    /// [`on_stop_notify`][Activation::on_stop_notify].
+    pub fn on_stop<F: 'static + Send + FnOnce(&mut Activation) -> ActorResult>(&mut self, action: F) {
+        if let Some(f) = self.active_facet() {
+            f.stop_actions.push(Box::new(action));
+        }
+    }
+
     /// Retrieve the [`Account`] against which actions are recorded.
     pub fn account(&self) -> &Arc<Account> {
         &self.pending.account
@@ -958,7 +1045,7 @@ impl<'activation> Activation<'activation> {
 
     /// Arranges for the active facet to be stopped cleanly when `self` commits.
     ///
-    /// Equivalent to `self.stop_facet(self.facet_id.unwrap(), None)`.
+    /// Equivalent to `self.stop_facet(self.facet.facet_id, None)`.
     pub fn stop(&mut self) {
         self.stop_facet(self.facet.facet_id, None)
     }
@@ -1015,6 +1102,132 @@ impl<'activation> Activation<'activation> {
             Ok(())
         }
     }
+
+    /// Create a new dataflow variable (field) within the active [`Actor`].
+    pub fn field<T: Any + Send>(&mut self, initial_value: T) -> Arc<Field<T>> {
+        let field_id = FieldId::new(NEXT_FIELD_ID.fetch_add(BUMP_AMOUNT.into(), Ordering::Relaxed))
+            .expect("Internal error: Attempt to allocate FieldId of zero. Too many FieldIds allocated. Restart the process.");
+        self.state.fields.insert(field_id, Box::new(initial_value));
+        Arc::new(Field {
+            field_id,
+            tx: self.state.tx.clone(),
+            phantom: PhantomData,
+        })
+    }
+
+    /// Retrieve a reference to the current value of a dataflow variable (field); if execution
+    /// is currently within a [dataflow block][Activation::dataflow], marks the block as
+    /// *depending upon* the field.
+    ///
+    pub fn get<T: Any + Send>(&mut self, field: &Field<T>) -> &T {
+        tracing::trace!(field = ?field.field_id, block = ?self.active_block, "get");
+        if let Some(block) = self.active_block {
+            self.state.dataflow.record_observation(block, field.field_id);
+        }
+        let any = self.state.fields.get(&field.field_id)
+            .expect("Attempt to get() missing field: wrong actor?");
+        any.downcast_ref().expect("Attempt to access field at incorrect type")
+    }
+
+    /// Retrieve a mutable reference to the contents of a dataflow variable (field). As for
+    /// [`get`][Activation::get], if used within a dataflow block, marks the block as
+    /// *depending upon* the field. In addition, because the caller may mutate the field, this
+    /// function (pessimistically) marks the field as dirty, which will lead to later
+    /// reevaluation of dependent blocks.
+    ///
+    pub fn get_mut<T: Any + Send>(&mut self, field: &Field<T>) -> &mut T {
+        tracing::trace!(field = ?field.field_id, block = ?self.active_block, "get_mut");
+        {
+            // Overapproximation.
+            if let Some(block) = self.active_block {
+                self.state.dataflow.record_observation(block, field.field_id);
+            }
+            self.state.dataflow.record_damage(field.field_id);
+        }
+        let any = self.state.fields.get_mut(&field.field_id)
+            .expect("Attempt to get_mut() missing field: wrong actor?");
+        any.downcast_mut().expect("Attempt to access field at incorrect type")
+    }
+
+    /// Overwrite the value of a dataflow variable (field). Marks the field as dirty, even if
+    /// the new value is [`eq`][std::cmp::PartialEq::eq] to the value being overwritten.
+    ///
+    pub fn set<T: Any + Send>(&mut self, field: &Field<T>, value: T) {
+        tracing::trace!(field = ?field.field_id, block = ?self.active_block, "set");
+        // Overapproximation in many cases, since the new value may not produce an
+        // observable difference (may be equal to the current value).
+        self.state.dataflow.record_damage(field.field_id);
+        let any = self.state.fields.get_mut(&field.field_id)
+            .expect("Attempt to set() missing field: wrong actor?");
+        *any = Box::new(value);
+    }
+
+    fn with_block(&mut self, block_id: BlockId, block: &mut Block) -> ActorResult {
+        let saved = self.active_block.replace(block_id);
+        let result = block(self);
+        self.active_block = saved;
+        result
+    }
+
+    /// Creates (and immediately executes) a new *dataflow block* that will be reexecuted if
+    /// any of its *dependent fields* (accessed via e.g. [`get`][Activation::get] or
+    /// [`get_mut`][Activation::get_mut]) are mutated.
+    ///
+    pub fn dataflow<F: 'static + Send + FnMut(&mut Activation) -> ActorResult>(&mut self, block: F) -> ActorResult {
+        let block_id = BlockId::new(NEXT_BLOCK_ID.fetch_add(BUMP_AMOUNT.into(), Ordering::Relaxed))
+            .expect("Internal error: Attempt to allocate BlockId of zero. Too many BlockIds allocated. Restart the process.");
+        let mut block: Block = Box::new(block);
+        self.with_block(block_id, &mut block)?;
+        self.state.blocks.insert(block_id, (self.facet.facet_id, block));
+        Ok(())
+    }
+
+    fn repair_dataflow(&mut self) -> Result<bool, Error> {
+        let mut pass_number = 0;
+        while !self.state.dataflow.is_clean() {
+            pass_number += 1;
+            tracing::trace!(?pass_number, "repair_dataflow");
+            let damaged_field_ids = self.state.dataflow.take_damaged_nodes();
+            for field_id in damaged_field_ids.into_iter() {
+                let block_ids = self.state.dataflow.take_observers_of(&field_id);
+                for block_id in block_ids.into_iter() {
+                    if let Some((facet_id, mut block)) = self.state.blocks.remove(&block_id) {
+                        let result = self.with_facet(
+                            true, facet_id, |t| t.with_block(block_id, &mut block));
+                        self.state.blocks.insert(block_id, (facet_id, block));
+                        result?;
+                    }
+                }
+            }
+        }
+        tracing::trace!(passes = ?pass_number, "repair_dataflow complete");
+        Ok(pass_number > 0)
+    }
+
+    fn _restore_invariants(&mut self) -> ActorResult {
+        loop {
+            loop {
+                let actions = std::mem::take(&mut self.pending.for_myself);
+                if actions.is_empty() { break; }
+                for action in actions.into_iter() { action(self)? }
+            }
+            if !self.repair_dataflow()? {
+                break;
+            }
+        }
+        Ok(())
+    }
+
+    fn restore_invariants(&mut self, d: RunDisposition) -> RunDisposition {
+        match d {
+            RunDisposition::Continue =>
+                self._restore_invariants().into(),
+            RunDisposition::Terminate(Ok(())) =>
+                RunDisposition::Terminate(self._restore_invariants()),
+            RunDisposition::Terminate(Err(_)) =>
+                d,
+        }
+    }
 }
 
 impl EventBuffer {
@@ -1200,6 +1413,9 @@ impl Actor {
             actor_id,
             tx,
             mailbox: Weak::new(),
+            dataflow: Graph::new(),
+            fields: HashMap::new(),
+            blocks: HashMap::new(),
             exit_hooks: Vec::new(),
             cleanup_actions: Map::new(),
             facet_nodes: Map::new(),
@@ -1261,7 +1477,7 @@ impl Actor {
 
         let terminate = |result: ActorResult| {
             let _ = root_facet_ref.activate_exit(Account::new(crate::name!("shutdown")),
-                                                 |_| Some(result));
+                                                 |_| RunDisposition::Terminate(result));
         };
 
         if root_facet_ref.activate(Account::new(crate::name!("boot")), boot).is_err() {
@@ -1279,15 +1495,18 @@ impl Actor {
                         tracing::trace!(actor_id = ?self.ac_ref.actor_id, "SystemMessage::Release");
                         return terminate(Ok(()));
                     }
+                    SystemMessage::ReleaseField(field_id) => {
+                        tracing::trace!(actor_id = ?self.ac_ref.actor_id,
+                                        "SystemMessage::ReleaseField({})", field_id);
+                        self.ac_ref.access(|s| if let ActorState::Running(ra) = s.unwrap() {
+                            ra.fields.remove(&field_id);
+                        })
+                    }
                     SystemMessage::Turn(mut loaned_item) => {
                         tracing::trace!(actor_id = ?self.ac_ref.actor_id, "SystemMessage::Turn");
-                        let mut actions = std::mem::take(&mut loaned_item.item);
+                        let actions = std::mem::take(&mut loaned_item.item);
                         let r = root_facet_ref.activate(Arc::clone(&loaned_item.account), |t| {
-                            loop {
-                                for action in actions.into_iter() { action(t)? }
-                                actions = std::mem::take(&mut t.pending.for_myself);
-                                if actions.is_empty() { break; }
-                            }
+                            for action in actions.into_iter() { action(t)? }
                             Ok(())
                         });
                         if r.is_err() { return; }
@@ -1338,8 +1557,8 @@ impl Facet {
     }
 }
 
-fn panicked_err() -> Option<ActorResult> {
-    Some(Err(error("Actor panicked", AnyValue::new(false))))
+fn panicked_err() -> ActorResult {
+    Err(error("Actor panicked", AnyValue::new(false)))
 }
 
 impl ActorRef {
@@ -1359,7 +1578,7 @@ impl ActorRef {
     /// abnormally.
     pub fn exit_status(&self) -> Option<ActorResult> {
         self.access(|s| s.map_or_else(
-            panicked_err,
+            || Some(panicked_err()),
             |state| match state {
                 ActorState::Running(_) => None,
                 ActorState::Terminated { exit_status } => Some((**exit_status).clone()),
@@ -1452,6 +1671,20 @@ impl RunningActor {
     }
 }
 
+impl<T: Any + Send> Eq for Field<T> {}
+impl<T: Any + Send> PartialEq for Field<T> {
+    fn eq(&self, other: &Field<T>) -> bool {
+        self.field_id == other.field_id
+    }
+}
+
+impl<T: Any + Send> Drop for Field<T> {
+    fn drop(&mut self) {
+        let _ = self.tx.send(SystemMessage::ReleaseField(self.field_id));
+        ()
+    }
+}
+
 impl Drop for Actor {
     fn drop(&mut self) {
         self.rx.close();
@@ -1555,15 +1788,18 @@ impl Cap {
     /// `AnyValue`, yields a `Cap` for the referenced entity. The
     /// `Cap` automatically decodes presented `AnyValue`s into
     /// instances of `M`.
-    pub fn guard<M: 'static + Send>(underlying: &Arc<Ref<M>>) -> Arc<Self>
+    pub fn guard<L: 'static + Sync + Send, M: 'static + Send>(
+        literals: Arc<L>,
+        underlying: Arc<Ref<M>>,
+    ) -> Arc<Self>
     where
-        for<'a> &'a M: Into<AnyValue>,
-        for<'a> M: TryFrom<&'a AnyValue>,
+        M: for<'a> Unparse<&'a L, AnyValue>,
+        M: for<'a> Parse<&'a L, AnyValue>,
     {
         Self::new(&Arc::new(Ref {
             mailbox: Arc::clone(&underlying.mailbox),
             facet_id: underlying.facet_id,
-            target: Mutex::new(Some(Box::new(Guard { underlying: underlying.clone() }))),
+            target: Mutex::new(Some(Box::new(Guard { underlying: underlying, literals }))),
         }))
     }
 
@@ -1600,15 +1836,28 @@ impl Cap {
     /// Translates `m` into an `AnyValue`, passes it through
     /// [`rewrite`][Self::rewrite], and then
     /// [`assert`s][Activation::assert] it using the activation `t`.
-    pub fn assert<M: Into<AnyValue>>(&self, t: &mut Activation, m: M) -> Option<Handle> {
-        self.rewrite(m.into()).map(|m| t.assert(&self.underlying, m))
+    pub fn assert<L, M: Unparse<L, AnyValue>>(&self, t: &mut Activation, literals: L, m: &M) -> Option<Handle>
+    {
+        self.rewrite(m.unparse(literals)).map(|m| t.assert(&self.underlying, m))
+    }
+
+    /// `update` is to [`assert`] as [`Activation::update`] is to [`Activation::assert`].
+    pub fn update<L, M: Unparse<L, AnyValue>>(
+        &self,
+        t: &mut Activation,
+        handle: &mut Option<Handle>,
+        literals: L,
+        m: Option<&M>,
+    ) {
+        t.update(handle, &self.underlying, m.and_then(|m| self.rewrite(m.unparse(literals))))
     }
 
     /// Translates `m` into an `AnyValue`, passes it through
     /// [`rewrite`][Self::rewrite], and then sends it via method
     /// [`message`][Activation::message] on the activation `t`.
-    pub fn message<M: Into<AnyValue>>(&self, t: &mut Activation, m: M) {
-        if let Some(m) = self.rewrite(m.into()) {
+    pub fn message<L, M: Unparse<L, AnyValue>>(&self, t: &mut Activation, literals: L, m: &M)
+    {
+        if let Some(m) = self.rewrite(m.unparse(literals)) {
             t.message(&self.underlying, m)
         }
     }
@@ -1643,13 +1892,13 @@ impl std::convert::From<&Cap> for IOValue {
     }
 }
 
-impl<M> Entity<AnyValue> for Guard<M>
+impl<L: Sync + Send, M> Entity<AnyValue> for Guard<L, M>
 where
-    for<'a> &'a M: Into<AnyValue>,
-    for<'a> M: TryFrom<&'a AnyValue>,
+    M: for<'a> Unparse<&'a L, AnyValue>,
+    M: for<'a> Parse<&'a L, AnyValue>,
 {
     fn assert(&mut self, t: &mut Activation, a: AnyValue, h: Handle) -> ActorResult {
-        match M::try_from(&a) {
+        match M::parse(&*self.literals, &a) {
             Ok(a) => t.with_entity(&self.underlying, |t, e| e.assert(t, a, h)),
             Err(_) => Ok(()),
         }
@@ -1658,7 +1907,7 @@ where
         t.with_entity(&self.underlying, |t, e| e.retract(t, h))
     }
     fn message(&mut self, t: &mut Activation, m: AnyValue) -> ActorResult {
-        match M::try_from(&m) {
+        match M::parse(&*self.literals, &m) {
             Ok(m) => t.with_entity(&self.underlying, |t, e| e.message(t, m)),
             Err(_) => Ok(()),
         }
@@ -1681,6 +1930,12 @@ impl<M> Entity<M> for StopOnRetract {
     }
 }
 
+impl<F: Send + FnMut(&mut Activation) -> ActorResult> Entity<Synced> for F {
+    fn message(&mut self, t: &mut Activation, _m: Synced) -> ActorResult {
+        self(t)
+    }
+}
+
 /// A convenient Syndicate-enhanced variation on
 /// [`tracing::info_span`].
 ///
@@ -1697,3 +1952,18 @@ macro_rules! name {
                                            task_id = tracing::field::Empty,
                                            oid = tracing::field::Empty)}
 }
+
+/// A convenient way of cloning a bunch of state shared among [entities][Entity], actions,
+/// linked tasks, etc.
+///
+/// Directly drawn from the discussion [here](https://github.com/rust-lang/rfcs/issues/2407).
+///
+#[macro_export]
+macro_rules! enclose {
+    ( ( $($name:ident),* ) $closure:expr ) => {
+        {
+            $(let $name = $name.clone();)*
+            $closure
+        }
+    }
+}
