@@ -1,12 +1,13 @@
 use bytes::Buf;
 use bytes::BytesMut;
 
+use crate::language;
 use crate::actor::*;
 use crate::during;
 use crate::error::Error;
 use crate::error::error;
 use crate::schemas::gatekeeper;
-use crate::schemas::internal_protocol as P;
+use crate::schemas::protocol as P;
 use crate::schemas::sturdy;
 
 use futures::Sink;
@@ -29,8 +30,9 @@ use preserves::value::ViaCodec;
 use preserves::value::Writer;
 use preserves::value::signed_integer::SignedInteger;
 
-use preserves_schema::support::Deserialize;
-use preserves_schema::support::ParseError;
+use preserves_schema::Codec;
+use preserves_schema::Deserialize;
+use preserves_schema::ParseError;
 
 use std::io;
 use std::pin::Pin;
@@ -91,7 +93,7 @@ pub struct TunnelRelay
     inbound_assertions: Map</* remote */ P::Handle, (/* local */ Handle, Vec<Arc<WireSymbol>>)>,
     outbound_assertions: Map<P::Handle, Vec<Arc<WireSymbol>>>,
     membranes: Membranes,
-    pending_outbound: Vec<P::TurnEvent>,
+    pending_outbound: Vec<P::TurnEvent<AnyValue>>,
     self_entity: Arc<Ref<()>>,
     output: UnboundedSender<LoanedItem<Vec<u8>>>,
     output_text: bool,
@@ -191,7 +193,7 @@ pub fn connect_stream<I, O, E, F>(
         let denotation = a.value().to_embedded()?;
         f(state, t, Arc::clone(denotation))
     }));
-    gatekeeper.assert(t, &gatekeeper::Resolve {
+    gatekeeper.assert(t, language(), &gatekeeper::Resolve {
         sturdyref,
         observer: Cap::new(&main_entity),
     });
@@ -249,7 +251,7 @@ impl TunnelRelay {
         result
     }
 
-    fn deserialize_one(&mut self, t: &mut Activation, bs: &[u8]) -> (Result<P::Packet, ParseError>, usize) {
+    fn deserialize_one(&mut self, t: &mut Activation, bs: &[u8]) -> (Result<P::Packet<AnyValue>, ParseError>, usize) {
         let mut src = BytesBinarySource::new(&bs);
         let mut dec = ActivatedMembranes {
             turn: t,
@@ -259,13 +261,13 @@ impl TunnelRelay {
         match src.peek() {
             Ok(v) => if v >= 128 {
                 self.output_text = false;
-                let mut r = src.packed::<_, AnyValue, _>(&mut dec);
+                let mut r = src.packed(&mut dec);
                 let res = P::Packet::deserialize(&mut r);
                 (res, r.source.index)
             } else {
                 self.output_text = true;
                 let mut dec = ViaCodec::new(dec);
-                let mut r = src.text::<_, AnyValue, _>(&mut dec);
+                let mut r = src.text::<AnyValue, _>(&mut dec);
                 let res = P::Packet::deserialize(&mut r);
                 (res, r.source.index)
             },
@@ -293,7 +295,7 @@ impl TunnelRelay {
         }
     }
 
-    fn handle_inbound_packet(&mut self, t: &mut Activation, p: P::Packet) -> ActorResult {
+    fn handle_inbound_packet(&mut self, t: &mut Activation, p: P::Packet<AnyValue>) -> ActorResult {
         tracing::trace!(packet = ?p, "-->");
         match p {
             P::Packet::Error(b) => {
@@ -310,7 +312,7 @@ impl TunnelRelay {
                             ws.inc_ref(),
                         None => {
                             tracing::warn!(
-                                event = ?AnyValue::from(&P::TurnEvent { oid, event }),
+                                event = ?language().unparse(&P::TurnEvent { oid, event }),
                                 "Cannot deliver event: nonexistent oid");
                             return Ok(());
                         }
@@ -322,7 +324,7 @@ impl TunnelRelay {
                             let P::Assert { assertion: P::Assertion(a), handle: remote_handle } = *b;
                             a.foreach_embedded::<_, Error>(
                                 &mut |r| Ok(pins.push(self.membranes.lookup_ref(r))))?;
-                            if let Some(local_handle) = target.assert(t, a) {
+                            if let Some(local_handle) = target.assert(t, &(), &a) {
                                 if let Some(_) = self.inbound_assertions.insert(remote_handle, (local_handle, pins)) {
                                     return Err(error("Assertion with duplicate handle", AnyValue::new(false)));
                                 }
@@ -334,7 +336,7 @@ impl TunnelRelay {
                         P::Event::Retract(b) => {
                             let P::Retract { handle: remote_handle } = *b;
                             let (local_handle, previous_pins) = match self.inbound_assertions.remove(&remote_handle) {
-                                None => return Err(error("Retraction of nonexistent handle", AnyValue::from(&remote_handle))),
+                                None => return Err(error("Retraction of nonexistent handle", language().unparse(&remote_handle))),
                                 Some(wss) => wss,
                             };
                             self.membranes.release(previous_pins);
@@ -353,7 +355,7 @@ impl TunnelRelay {
                                     _ => Ok(())
                                 }
                             })?;
-                            target.message(t, a);
+                            target.message(t, &(), &a);
                             self.membranes.release(pins);
                             dump_membranes!(self.membranes);
                         }
@@ -368,7 +370,7 @@ impl TunnelRelay {
                             }
                             impl Entity<Synced> for SyncPeer {
                                 fn message(&mut self, t: &mut Activation, _a: Synced) -> ActorResult {
-                                    self.peer.message(t, AnyValue::new(true));
+                                    self.peer.message(t, &(), &AnyValue::new(true));
                                     let mut g = self.relay_ref.lock().expect("unpoisoned");
                                     let tr = g.as_mut().expect("initialized");
                                     tr.membranes.release(std::mem::take(&mut self.pins));
@@ -393,9 +395,9 @@ impl TunnelRelay {
 
     fn outbound_event_bookkeeping(
         &mut self,
-        t: &mut Activation,
+        _t: &mut Activation,
         remote_oid: sturdy::Oid,
-        event: &P::Event,
+        event: &P::Event<AnyValue>,
     ) -> ActorResult {
         match event {
             P::Event::Assert(b) => {
@@ -434,23 +436,23 @@ impl TunnelRelay {
         Ok(())
     }
 
-    pub fn send_packet(&mut self, account: &Arc<Account>, cost: usize, p: P::Packet) -> ActorResult {
-        let item = AnyValue::from(&p);
+    pub fn send_packet(&mut self, account: &Arc<Account>, cost: usize, p: P::Packet<AnyValue>) -> ActorResult {
+        let item = language().unparse(&p);
         tracing::trace!(packet = ?item, "<--");
 
         let bs = if self.output_text {
-            let mut s = TextWriter::encode::<_, AnyValue, _>(&mut self.membranes, &item)?;
+            let mut s = TextWriter::encode(&mut self.membranes, &item)?;
             s.push('\n');
             s.into_bytes()
         } else {
-            PackedWriter::encode::<_, AnyValue, _>(&mut self.membranes, &item)?
+            PackedWriter::encode(&mut self.membranes, &item)?
         };
 
         let _ = self.output.send(LoanedItem::new(account, cost, bs));
         Ok(())
     }
 
-    pub fn send_event(&mut self, t: &mut Activation, remote_oid: sturdy::Oid, event: P::Event) -> ActorResult {
+    pub fn send_event(&mut self, t: &mut Activation, remote_oid: sturdy::Oid, event: P::Event<AnyValue>) -> ActorResult {
         if self.pending_outbound.is_empty() {
             t.message_for_myself(&self.self_entity, ());
         }
@@ -515,7 +517,7 @@ impl Membranes {
         relay_ref: &TunnelRelayRef,
         src: &'src mut S,
         _read_annotations: bool,
-    ) -> io::Result<P::_Ptr> {
+    ) -> io::Result<Arc<Cap>> {
         let ws = match sturdy::WireRef::deserialize(&mut src.packed(NoEmbeddedDomainCodec))? {
             sturdy::WireRef::Mine{ oid: b } => {
                 let oid = *b;
@@ -547,23 +549,23 @@ impl Membranes {
     }
 }
 
-impl<'a, 'activation, 'm> DomainDecode<P::_Ptr> for ActivatedMembranes<'a, 'activation, 'm> {
+impl<'a, 'activation, 'm> DomainDecode<Arc<Cap>> for ActivatedMembranes<'a, 'activation, 'm> {
     fn decode_embedded<'de, 'src, S: BinarySource<'de>>(
         &mut self,
         src: &'src mut S,
         read_annotations: bool,
-    ) -> io::Result<P::_Ptr> {
+    ) -> io::Result<Arc<Cap>> {
         self.membranes.decode_embedded(self.turn, self.tr_ref, src, read_annotations)
     }
 }
 
-impl DomainEncode<P::_Ptr> for Membranes {
+impl DomainEncode<Arc<Cap>> for Membranes {
     fn encode_embedded<W: Writer>(
         &mut self,
         w: &mut W,
-        d: &P::_Ptr,
+        d: &Arc<Cap>,
     ) -> io::Result<()> {
-        w.write(&mut NoEmbeddedDomainCodec, &AnyValue::from(&match self.exported.ref_map.get(d) {
+        w.write(&mut NoEmbeddedDomainCodec, &language().unparse(&match self.exported.ref_map.get(d) {
             Some(ws) => sturdy::WireRef::Mine {
                 oid: Box::new(ws.inc_ref().oid.clone()),
             },
@@ -711,7 +713,7 @@ impl Entity<AnyValue> for RelayEntity {
         let mut g = self.relay_ref.lock().expect("unpoisoned");
         let tr = g.as_mut().expect("initialized");
         tr.send_event(t, self.oid.clone(), P::Event::Sync(Box::new(P::Sync {
-            peer: Cap::guard(&peer)
+            peer: Cap::guard(Arc::new(()), peer)
         })))
     }
 }
